package game;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.List;
import javax.microedition.midlet.*;
import javax.microedition.rms.RecordEnumeration;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import ui.Table;

/*
 * Az alkalmazas MIDlet osztalya.
 */
public class NumbersMidlet extends MIDlet implements CommandListener {
    private Display disp;       //keprenyot reprezentalo objektum
    private List menu;          //fomenu
    private List list;          //nehezsegi szint valaszto kepernyo
    private Form records;       //rekordokat tartalmazo kepernyo
    private Form help;          //a help
    private Command exit;
    private Command ok;
    private Command back;
    private Command start;

    private Table canvas;
    private static final String recStore = "NumbersRecStore";

    public NumbersMidlet() {
        //elemek inicializalasa
        exit = new Command("Exit", Command.EXIT, 0);
        ok = new Command("Ok", Command.OK, 0);
        back = new Command("Back", Command.BACK, 0);
        start = new Command("Start", Command.OK, 0);

        menu = new List("Numbers", List.IMPLICIT);
        menu.append("Game", null);
        menu.append("Records", null);
        menu.append("Help", null);
        menu.addCommand(exit);
        menu.addCommand(ok);

        list = new List("Difficulty", List.EXCLUSIVE);
        list.append("Small", null);
        list.append("Medium", null);
        list.append("Large", null);
        list.addCommand(back);
        list.addCommand(start);

        records = new Form("Records");
        records.addCommand(back);

        help = new Form("Help");
        help.append("This is help...");
        help.addCommand(back);
    }

    //alkalmazas inditasa
    public void startApp() {
        menu.setCommandListener(this);
        list.setCommandListener(this);
        records.setCommandListener(this);
        help.setCommandListener(this);

        disp = Display.getDisplay(this);
        disp.setCurrent(menu);
    }

    public void pauseApp() {
    }

    //alkalmazas bezarasa
    public void destroyApp(boolean unconditional) {
        disp.setCurrent(null);
        notifyDestroyed();
    }

    //parancsok kezelese
    public void commandAction(Command c, Displayable d) {
        //kilepes
        if (c == exit) {
            //ha a fomenubol lep ki, bezarja az alkalmazast, egyebkent visszalep a fomenube
            if (d == menu) destroyApp(true);
            else if (d == canvas){
                disp.setCurrent(menu);
                canvas = null;
            }
        }
        //nyugtazas (csak a fomenuhoz vaan hozzarendelve)
        else if (c == ok) {
            //attol fuggoen, hogy a fomenu melyik elemen allunk, behozza az aktualis keprenyot
            switch (menu.getSelectedIndex()) {
                case 0: disp.setCurrent(list); break;
                case 1: fillRecords(); disp.setCurrent(records); break;
                case 2: disp.setCurrent(help); break;
                default: break;
            }
        }
        else if (c == back) {
            disp.setCurrent(menu);
        }
        //jatek inditasa
        else if (c == start) {
            //ha a nehezsegi szint valaszto keprenyon allunk, letrehozza a jatek Canvas-t
            if (d == list) {
                int size = list.getSelectedIndex() + 3;

                canvas = new Table(disp, size);
                canvas.mix();
                canvas.addCommand(exit);
                canvas.addCommand(start);
                canvas.setCommandListener(this);
                disp.setCurrent(canvas);
            }
            //ha a jatek Canvas-on allunk, uj osztast ad
            else if (d == canvas){
                canvas.mix();
            }
        }
    }

    //rekordok betoltese a RecordStore-bol (RMS)
    private void fillRecords() {
        try {
            records.deleteAll();

            //az alkalmazashoz tartozo RecordStore megnyitasa
            RecordStore rs = RecordStore.openRecordStore(recStore, true);   //true -> ha nincs ilyen nevu RS, akkor letrehozza azt

            //a RS-ban levo rekordok lekerdezese
            if (rs.getNumRecords() > 0) {
                RecordEnumeration re = rs.enumerateRecords(null, null, false);  //a rekordok bejarasahoz szukseges RecordEnumeration objektum
                int[] rec = new int[rs.getNumRecords()];
                int k = 0;

                //amig a RS-ban van kovetkezo elem, berakjuk azt egy int tombbe (rekordok tombje)
                while (re.hasNextElement()) {
                    rec[k] = Integer.parseInt(new String(re.nextRecord()));
                    k++;
                }

                //RS bezarasa (kell!!!)
                rs.closeRecordStore();
                rs = null;

                records.append(new StringBuffer("Small: " + ((rec[0] == 10000) ? "-" : String.valueOf(rec[0])) + "\n").toString());
                records.append(new StringBuffer("Medium: -" + ((rec[1] == 10000) ? "-" : String.valueOf(rec[1])) + "\n").toString());
                records.append(new StringBuffer("Large: -" + ((rec[2] == 10000) ? "-" : String.valueOf(rec[2])) + "\n").toString());
            }
            else {
                records.append("Small: -\n");
                records.append("Medium: -\n");
                records.append("Large: -\n");
            }
        } catch (RecordStoreException ex) {
            ex.printStackTrace();
        }
    }
}
