package ui;

import java.util.Random;
import java.util.Vector;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Graphics;
import javax.microedition.rms.RecordEnumeration;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;

/*
 * A jatekteret megvalosito osztaly.
 */
public class Table extends Canvas {
    private final int size;     //a jatektabla merete
    private int steps;          //hanyszor tortent csere
    private int actual;         //melyik a kijelolt mezo
    private Vector fields;      //mezok

    private boolean running;    //vege-e a jateknak
    private final Alert alert;
    private final Display disp;

    private static final String recStore = "NumbersRecStore";

    public Table(final Display d, final int size) {
        this.size = size;
        init();

        running = true;
        alert = new Alert("Congratulations!", "", null, AlertType.CONFIRMATION);
        disp = d;
    }

    //mezok inicializalasa, vektor feltoltese
    private void init() {
        steps = 0;
        fields = new Vector(size * size + 1);
        for (int i = 0; i < size; i++) {
            for (int j = 0; j < size; j++) {
                Field field = new Field(i, j);
                field.setNumber(steps++);
                fields.addElement(field);
            }
        }

        steps = 0;
        actual = 0;
    }

    //mezok osszekeverese
    public void mix() {
        Random rand = new Random();
        for (int i = 0; i < fields.size(); i++) {
            int n = rand.nextInt(fields.size());
            if (n != i) {
                replace(n, i);
            }
        }

        rand = null;
        steps = 0;
        running = true;
        repaint();
    }

    //ket mezo csereje (ill. a rajtuk levo szamok csereje)
    private void replace(int n, int m) {
        int num = ((Field)fields.elementAt(n)).getNumber();
        ((Field)fields.elementAt(n)).setNumber(((Field)fields.elementAt(m)).getNumber());
        ((Field)fields.elementAt(m)).setNumber(num);
    }

    //a kulonbozo iranyokba valo lepesek (a kijelolt mezo atallitasa)
    private void left() {
        if (actual == 0 || actual % size == 0) actual += (size - 1);
        else actual -= 1;
    }

    private void right() {
        if (actual % size == size - 1) actual -= (size - 1);
        else actual += 1;
    }

    private void up() {
        actual -= size;
        if (actual < 0) actual = fields.size() + actual;
    }

    private void down() {
        actual += size;
        if (actual > fields.size() - 1) actual = actual - fields.size();
    }

    //csere muvelete (csak egymas melleti mezok cserelhetok, es az egyiknek az ures mezonek kell lennie)
    private void move() {
        int to = findEmpty();   //az ures mezo vektorbeli indexe
        if ((actual != 0 && actual % size != 0 && actual - 1 == to) ||
            (actual % size != size - 1 && actual + 1 == to) ||
            (actual - size == to) || (actual + size) == to) {

            replace(to, actual);
            steps++;

            repaint();

            //jatek vegenek kezelese
            if (isEnd()) {
                running = false;
                alert.setString("Steps: " + steps);
                
                try {
                    //az eddigi rekor megdontesenek ellenorzese
                    if (checkResults()) alert.setString("Steps: " + steps + "\nNew record!!!");
                } catch (RecordStoreException ex) {
                    ex.printStackTrace();
                }

                disp.setCurrent(alert, this);
            }
        }
    }

    //megkeresi a vektorban az ures (0-dik) mezot
    private int findEmpty() {
        int n = 0;
        for (int i = 0; i < fields.size(); i++) {
            if (((Field)fields.elementAt(i)).getNumber() == 0) {
                n = i;
                break;
            }
        }

        return n;
    }

    //ellenorzi, hogy a jatek veget ert-e (vaygis az ures mezo nelkul sorrendben vannak-e a mezok szamai)
    private boolean isEnd() {
        int n = 0;
        int m = 0;
        for (int i = 1; i < fields.size(); i++) {
            n = ((Field)fields.elementAt(i - 1)).getNumber();
            m = ((Field)fields.elementAt(i)).getNumber();

            if (m != 0 && n > m ) return false;
            if (n == 0 && i > 1 && ((Field)fields.elementAt(i - 2)).getNumber() > m) return false;
        }

        return true;
    }

    //gombnyomas feluldefinialasa
    public void keyPressed(int keycode) {
        if (keycode >= KEY_NUM2 && keycode <= KEY_NUM8) {
            switch(keycode) {
                case KEY_NUM2: up(); break;
                case KEY_NUM4: left(); break;
                case KEY_NUM5: if (running) move(); break; //csere csak akkor engedelyezett, ha meg megy a jatek
                case KEY_NUM6: right(); break;
                case KEY_NUM8: down(); break;
                default: break;
            }
        }
        else {
            int gc = getGameAction(keycode);
            switch(gc) {
                case UP: up(); break;
                case DOWN: down(); break;
                case LEFT: left(); break;
                case RIGHT: right(); break;
                case FIRE: if (running) move(); break;
                default: break;
            }
        }

        repaint();
    }

    //kirajzolas
    protected void paint(Graphics g) {
        g.setColor(255, 255, 255);
        g.fillRect(0, 0, getWidth(), getHeight());

        int m = getWidth()/size;
        for (int i = 0; i < fields.size(); i++) {
            if (i == actual) ((Field)fields.elementAt(i)).paint(g, m, true);
            else ((Field)fields.elementAt(i)).paint(g, m, false);
        }

        g.setColor(0, 0, 0);
        g.drawString("Steps: " + String.valueOf(steps), 2, getHeight() - 30, 0);
    }

    //uj rekord ellenorzese es kezelese (RMS-sel)
    private boolean checkResults() throws RecordStoreException {
        //a jatekhoz tartozo RecordStore megnyitasa
        RecordStore rs = RecordStore.openRecordStore(recStore, true); //true -> ha nincs ilyen nevu RS, akkor letrehozza azt

        //a RS-ban levo rekordok lekerdezese
        if (rs.getNumRecords() > 0) {
            RecordEnumeration re = rs.enumerateRecords(null, null, false); //a rekordok bejarasahoz szukseges RecordEnumeration objektum
            String[] records = new String[rs.getNumRecords()];
            int k = 0;

            //amig a RS-ban van kovetkezo elem, berakjuk azt egy String tombbe
            while (re.hasNextElement()) {
                records[k] = new String(re.nextRecord());
                k++;
            }
            rs.closeRecordStore();
            rs = null;

            //ellenorzi, hogy a szintnek megfelelo rekord rosszabb-e a most elertnel
            if (Integer.parseInt(records[size - 3]) > steps) {
                records[size - 3] = String.valueOf(steps);          //a megfelelo szinten beallitja az uj rekordot
                //RecordStore.deleteRecordStore(recStore);            //torli az eddig tarolt rekordokat (mivel atirni nem tudjuk az adott rekordot, erre szukseg van)
                rs = RecordStore.openRecordStore(recStore, true);   //ujra letrehozza a RS-t
                
                byte[] data = records[size - 3].getBytes();
                rs.setRecord(size - 3 + 1, data, 0, data.length);

                //a RS feltoltese a friss adatokkal
                /*for (int i = 0; i < records.length; i++) {
                    byte[] data = records[i].getBytes();
                    System.out.println(rs.getNextRecordID());
                    rs.addRecord(data, 0, data.length);     //rekord felvete (mit, melyik indextol, meddig)
                }*/

                //RS bezarasa (kell!!!)
                rs.closeRecordStore();
                rs = null;

                return true;
            }
        }
        else {
            //ha nincsenek rekordok a RS-ban, akkor inicializaljuk azokat, es amegfelelot kicsereljuk az aktualis rekordra
            String[] records = {"10000", "10000", "10000"};
            records[size - 3] = String.valueOf(steps);
            for (int i = 0; i < records.length; i++) {
                byte[] data = records[i].getBytes();
                rs.addRecord(data, 0, data.length);
            }

            rs.closeRecordStore();
            rs = null;

            return true;
        }

        if (rs != null) {
            rs.closeRecordStore();
            rs = null;
        }
        
        return false;
    }
}
